# coding: utf-8
u"""Пакет тестов для m3-ui.

Тесты запускаются командами:

* все тесты

  ./run.py

* тесты конкретного приложения

  ./run.sh m3_ext.ui

* тесты нескольких конкретных приложений

  ./run.sh m3_ext.ui
"""
